﻿Imports System.ComponentModel
Imports System.Collections.ObjectModel
Imports System.Linq

Public Class MainVM
    Implements INotifyPropertyChanged

#Region "Collections"
    Private _nodes As New ObservableCollection(Of Node)
    Public ReadOnly Property Nodes As ObservableCollection(Of Node)
        Get
            Return _nodes
        End Get
    End Property

    Private _elements As New ObservableCollection(Of Element)
    Public ReadOnly Property Elements As ObservableCollection(Of Element)
        Get
            Return _elements
        End Get
    End Property
#End Region

    Public Sub New()
        ShowAllCoordinates = True
        ShowNames = True
    End Sub

    Private _selectedObject As DesignerObject
    Public Property SelectedObject As DesignerObject
        Get
            Return _selectedObject
        End Get
        Set(value As DesignerObject)
            For Each item In Nodes
                item.IsHighlighted = False
            Next
            _selectedObject = value
            OnPropertyChanged("SelectedObject")
            'DeleteCommand.IsEnabled = value != null;
            Dim element = TryCast(value, Element)
            If element IsNot Nothing Then
                If element.StartingNode IsNot Nothing Then
                    element.StartingNode.IsHighlighted = True
                End If
                If element.EndingNode IsNot Nothing Then
                    element.EndingNode.IsHighlighted = True
                End If
            End If
        End Set
    End Property

#Region "Creating new DesignerObjects"
    Private _creatingNewNode As Boolean
    Public Property CreatingNewNode As Boolean
        Get
            Return _creatingNewNode
        End Get
        Set(value As Boolean)
            _creatingNewNode = value
            OnPropertyChanged("CreatingNewNode")
            If value Then
                CreateNewNode()
            Else
                RemoveNewObjects()
            End If
        End Set
    End Property
    Public Sub CreateNewNode()
        Dim newNode As New Node With {.ID = "node" & (Nodes.Count + 1).ToString, .IsNew = True}
        Nodes.Add(newNode)
        SelectedObject = newNode
    End Sub


    Private _creatingNewElement As Boolean
    Public Property CreatingNewElement As Boolean
        Get
            Return _creatingNewElement
        End Get
        Set(value As Boolean)
            _creatingNewElement = value
            OnPropertyChanged("CreatingNewElement")
            If value Then
                CreateNewElement()
            Else
                RemoveNewObjects()
            End If
        End Set
    End Property
    Public Sub CreateNewElement()
        Dim newElement As New Element With {.ID = "elem" & (Elements.Count + 1).ToString, .IsNew = True}
        Elements.Add(newElement)
        SelectedObject = newElement
    End Sub

    Public Sub RemoveNewObjects()
        Dim tempList1 As New List(Of Node)
        For Each item In Nodes
            If item.IsNew Then tempList1.Add(item)
        Next
        For Each item In tempList1
            Nodes.Remove(item)
        Next

        Dim tempList2 As New List(Of Element)
        For Each item In Elements
            If item.IsNew Then tempList2.Add(item)
        Next
        For Each item In tempList2
            Elements.Remove(item)
        Next
        'Nodes.Where(x >= x.IsNew).ToList().ForEach(x >= Nodes.Remove(x))

        'Connectors.Where(x => x.IsNew).ToList().ForEach(x => Connectors.Remove(x));
    End Sub
#End Region




#Region "Visibility"

    Private _showNames As Boolean
    Public Property ShowNames As Boolean
        Get
            Return _showNames
        End Get
        Set(value As Boolean)
            _showNames = value
            OnPropertyChanged("ShowNames")
        End Set
    End Property

    Private _showCurrentCoordinates As Boolean
    Public Property ShowCurrentCoordinates As Boolean
        Get
            Return _showCurrentCoordinates
        End Get
        Set(value As Boolean)
            _showCurrentCoordinates = value
            OnPropertyChanged("ShowCurrentCoordinates")
        End Set
    End Property
    Private _showAllCoordinates As Boolean
    Public Property ShowAllCoordinates As Boolean
        Get
            Return _showAllCoordinates
        End Get
        Set(value As Boolean)
            _showAllCoordinates = value
            OnPropertyChanged("ShowAllCoordinates")
        End Set
    End Property

#End Region
    Private Sub Delete()
        If TypeOf SelectedObject Is Element Then
            Dim item As Element = TryCast(SelectedObject, Element)
            Elements.Remove(item)
        ElseIf TypeOf SelectedObject Is Node Then
            Dim node As Node = TryCast(SelectedObject, Node)
            For Each ele In Elements
                If ele.StartingNode.Equals(node) Then Elements.Remove(ele)
                If ele.EndingNode.Equals(node) Then Elements.Remove(ele)
            Next
            Nodes.Remove(node)
        End If

    End Sub
    Public Event PropertyChanged(sender As Object, e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged
    Protected Sub OnPropertyChanged(ByVal name As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(name))
    End Sub


#Region "Scrolling support"

    Private _areaHeight As Double = 500
    Public Property AreaHeight As Double
        Get
            Return _areaHeight
        End Get
        Set(value As Double)
            _areaHeight = value
            OnPropertyChanged("AreaHeight")
        End Set
    End Property

    Private _areaWidth As Double = 500
    Public Property AreaWidth As Double
        Get
            Return _areaWidth
        End Get
        Set(value As Double)
            _areaWidth = value
            OnPropertyChanged("AreaWidth")
        End Set
    End Property
#End Region





End Class

'private Command _deleteCommand;
'public Command DeleteCommand


